@@ -25,7 +25,7 @@ Constraint that checks against the unicode C<print> character class,
 Is a sub-class of, and inherits methods from L<HTML::FormFu::Constraint::Regex>,
 L<HTML::FormFu::Constraint>
 
-L<HTML::FormFu::FormFu>
+L<HTML::FormFu>
 
 =head1 AUTHOR
 
