@@ -10,11 +10,20 @@ use DateTime::Format::Builder;
 use DateTime::Format::Natural;
 use DateTime::Locale;
 use Scalar::Util qw( blessed );
+use List::MoreUtils qw( all none uniq );
 use Carp qw( croak );
 
-__PACKAGE__->mk_attrs( qw( day  month  year ) );
+__PACKAGE__->mk_attrs(qw( day  month  year ));
 
-__PACKAGE__->mk_item_accessors( qw( strftime auto_inflate default_natural ) );
+__PACKAGE__->mk_accessors( qw(
+        _known_fields
+) );
+
+__PACKAGE__->mk_item_accessors( qw(
+        strftime
+        auto_inflate
+        default_natural
+) );
 
 *default = \&value;
 
@@ -23,7 +32,7 @@ for my $method ( qw(
     deflator        filter
     constraint      inflator
     validator       transformer
-) )
+    ) )
 {
     my $sub = sub {
         my $self       = shift;
@@ -48,17 +57,21 @@ sub new {
     my $self = shift->next::method(@_);
 
     $self->strftime("%d-%m-%Y");
-    
+
+    $self->_known_fields( [qw/ day month year /] );
+
+    $self->field_order( [qw/ day month year /] );
+
     $self->day( {
             type   => '_MultiSelect',
             prefix => [],
         } );
-    
+
     $self->month( {
             type   => '_MultiSelect',
             prefix => [],
         } );
-    
+
     $self->year( {
             type   => '_MultiSelect',
             prefix => [],
@@ -81,9 +94,14 @@ sub value {
         if ( @{ $self->_elements } ) {
             $self->_date_defaults;
 
-            $self->_elements->[0]->default( $self->day->{default} );
-            $self->_elements->[1]->default( $self->month->{default} );
-            $self->_elements->[2]->default( $self->year->{default} );
+            my @order = @{ $self->field_order };
+
+            for my $i ( 0 .. $#order ) {
+                my $field = $order[$i];
+
+                $self->_elements->[$i]
+                    ->default( $value ? $self->$field->{default} : undef );
+            }
         }
 
         return $self;
@@ -93,15 +111,17 @@ sub value {
 }
 
 sub _add_elements {
-    my ( $self ) = @_;
+    my ($self) = @_;
 
     $self->_elements( [] );
 
     $self->_date_defaults;
 
-    $self->_add_day;
-    $self->_add_month;
-    $self->_add_year;
+    for my $order ( @{ $self->field_order } ) {
+        my $method = "_add_$order";
+
+        $self->$method;
+    }
 
     if ( $self->auto_inflate
         && !@{ $self->get_inflators( { type => "DateTime" } ) } )
@@ -113,10 +133,10 @@ sub _add_elements {
 }
 
 sub _date_defaults {
-    my ( $self ) = @_;
+    my ($self) = @_;
 
     my $default;
-    
+
     if ( defined( $default = $self->default_natural ) ) {
         my $parser = DateTime::Format::Natural->new;
         $default = $parser->parse_datetime($default);
@@ -133,9 +153,9 @@ sub _date_defaults {
     }
 
     if ( defined $default ) {
-        $self->day->{default}   = $default->day;
-        $self->month->{default} = $default->month;
-        $self->year->{default}  = $default->year;
+        for my $field ( @{ $self->field_order } ) {
+            $self->$field->{default} = $default->$field;
+        }
     }
 
     return;
@@ -146,21 +166,22 @@ sub _add_day {
 
     my $day = $self->day;
 
-    my $day_name = _build_day_name($self);
+    my $day_name = $self->_build_name('day');
+
+    my @day_prefix
+        = ref $day->{prefix}
+        ? @{ $day->{prefix} }
+        : $day->{prefix};
 
-    my @day_prefix = ref $day->{prefix} ? @{ $day->{prefix} }
-                   :                      $day->{prefix}
-                   ;
-    
     @day_prefix = map { [ '', $_ ] } @day_prefix;
 
     $self->element( {
-        type    => $day->{type},
-        name    => $day_name,
-        options => [ @day_prefix, map { [ $_, $_ ] } 1 .. 31 ],
-        
-        defined $day->{default} ? ( default => $day->{default} ) : (),
-    } );
+            type    => $day->{type},
+            name    => $day_name,
+            options => [ @day_prefix, map { [ $_, $_ ] } 1 .. 31 ],
+
+            defined $day->{default} ? ( default => $day->{default} ) : (),
+        } );
 
     return;
 }
@@ -170,14 +191,15 @@ sub _add_month {
 
     my $month = $self->month;
 
-    my $month_name = _build_month_name($self);
+    my $month_name = $self->_build_name('month');
 
     my @months = _build_month_list($self);
 
-    my @month_prefix = ref $month->{prefix} ? @{ $month->{prefix} }
-                     :                        $month->{prefix}
-                     ;
-    
+    my @month_prefix
+        = ref $month->{prefix}
+        ? @{ $month->{prefix} }
+        : $month->{prefix};
+
     @month_prefix = map { [ '', $_ ] } @month_prefix;
 
     my $options = [ @month_prefix, map { [ $_ + 1, $months[$_] ] } 0 .. 11 ];
@@ -186,7 +208,7 @@ sub _add_month {
             type    => $month->{type},
             name    => $month_name,
             options => $options,
-            
+
             defined $month->{default} ? ( default => $month->{default} ) : (),
         } );
 
@@ -198,28 +220,30 @@ sub _add_year {
 
     my $year = $self->year;
 
-    my $year_name = _build_year_name($self);
+    my $year_name = $self->_build_name('year');
 
-    my $year_ref = defined $year->{reference} ? $year->{reference}
-                 :                              ( localtime(time) )[5] + 1900
-                 ;
+    my $year_ref
+        = defined $year->{reference}
+        ? $year->{reference}
+        : ( localtime(time) )[5] + 1900;
 
     my @years
         = defined $year->{list}
         ? @{ $year->{list} }
         : ( $year_ref - $year->{less} ) .. ( $year_ref + $year->{plus} );
 
-    my @year_prefix = ref $year->{prefix} ? @{ $year->{prefix} }
-                    :                       $year->{prefix}
-                    ;
-    
+    my @year_prefix
+        = ref $year->{prefix}
+        ? @{ $year->{prefix} }
+        : $year->{prefix};
+
     @year_prefix = map { [ '', $_ ] } @year_prefix;
 
     $self->element( {
             type    => $year->{type},
             name    => $year_name,
             options => [ @year_prefix, map { [ $_, $_ ] } @years ],
-            
+
             defined $year->{default} ? ( default => $year->{default} ) : (),
         } );
 
@@ -240,18 +264,18 @@ sub _build_month_list {
         if ( ref $languages ne 'ARRAY' ) {
             $languages = [$languages];
         }
-        
-        for my $lang ( @$languages ) {
+
+        for my $lang (@$languages) {
             my $loc;
-            
+
             eval { $loc = DateTime::Locale->load($lang) };
             if ( !$@ ) {
                 @months
-                    = $month->{short_names} ? @{ $loc->month_abbreviations }
-                    :                         @{ $loc->month_names }
-                    ;
+                    = $month->{short_names}
+                    ? @{ $loc->month_abbreviations }
+                    : @{ $loc->month_names };
 
-                @months = map { ucfirst } @months;
+                @months = map {ucfirst} @months;
 
                 last;
             }
@@ -261,37 +285,15 @@ sub _build_month_list {
     return @months;
 }
 
-sub _build_day_name {
-    my ($self) = @_;
+sub _build_name {
+    my ( $self, $type ) = @_;
 
-    my $day_name
-        = defined $self->day->{name}
-        ? $self->day->{name}
-        : sprintf "%s_day", $self->name;
+    my $name
+        = defined $self->$type->{name}
+        ? $self->$type->{name}
+        : sprintf "%s_%s", $self->name, $type;
 
-    return $day_name;
-}
-
-sub _build_month_name {
-    my ($self) = @_;
-
-    my $month_name
-        = defined $self->month->{name}
-        ? $self->month->{name}
-        : sprintf "%s_month", $self->name;
-
-    return $month_name;
-}
-
-sub _build_year_name {
-    my ($self) = @_;
-
-    my $year_name
-        = defined $self->year->{name}
-        ? $self->year->{name}
-        : sprintf "%s_year", $self->name;
-
-    return $year_name;
+    return $name;
 }
 
 sub _add_inflator {
@@ -306,6 +308,31 @@ sub _add_inflator {
     return;
 }
 
+sub field_order {
+    my ( $self, @order ) = @_;
+
+    if ( @_ > 1 ) {
+        if ( @order == 1 && ref( $order[0] ) eq 'ARRAY' ) {
+            @order = @{ $order[0] };
+        }
+
+        for my $field (@order) {
+            croak "unknown field type: '$field'"
+                if none { $field eq $_ } @{ $self->_known_fields };
+        }
+
+        croak 'repeated field type'
+            if scalar( uniq @order ) != scalar(@order);
+
+        $self->{field_order} = \@order;
+
+        return $self;
+    }
+    else {
+        return $self->{field_order};
+    }
+}
+
 sub process {
     my ( $self, @args ) = @_;
 
@@ -317,33 +344,25 @@ sub process {
 sub process_input {
     my ( $self, $input ) = @_;
 
-    my $day_name   = _build_day_name($self);
-    my $month_name = _build_month_name($self);
-    my $year_name  = _build_year_name($self);
+    my %value;
 
-    $day_name   = $self->get_element( { name => $day_name }   )->nested_name;
-    $month_name = $self->get_element( { name => $month_name } )->nested_name;
-    $year_name  = $self->get_element( { name => $year_name }  )->nested_name;
+    my @order = @{ $self->field_order };
 
-    my $day   = $self->get_nested_hash_value( $input, $day_name );
-    my $month = $self->get_nested_hash_value( $input, $month_name );
-    my $year  = $self->get_nested_hash_value( $input, $year_name );
+    for my $i ( 0 .. $#order ) {
+        my $field = $order[$i];
 
-    if (   defined $day
-        && length $day
-        && defined $month
-        && length $month
-        && defined $year
-        && length $year )
+        my $name = $self->_elements->[$i]->nested_name;
+
+        $value{$field} = $self->get_nested_hash_value( $input, $name );
+    }
+
+    if ( ( all {defined} values %value )
+        && all {length} values %value )
     {
         my $dt;
 
         eval {
-            $dt = DateTime->new(
-                day   => $day,
-                month => $month,
-                year  => $year,
-            );
+            $dt = DateTime->new( map { $_, $value{$_} } keys %value );
         };
 
         my $value;
@@ -369,9 +388,9 @@ sub render_data_non_recursive {
     my ( $self, $args ) = @_;
 
     my $render = $self->next::method( {
-        elements => [ map { $_->render_data } @{ $self->_elements } ],
-        $args ? %$args : (),
-    } );
+            elements => [ map { $_->render_data } @{ $self->_elements } ],
+            $args ? %$args : (),
+        } );
 
     return $render;
 }
@@ -573,6 +592,16 @@ Arguments: $count
 
 Default Value: 10
 
+=head2 field_order
+
+Arguments: \@fields
+
+Default Value: ['day', 'month', 'year']
+
+Specify the order of the date fields in the rendered HTML.
+
+Not all 3 fields are required. No single field can be used more than once.
+
 =head2 auto_inflate
 
 If true, a L<DateTime Inflator|HTML::FormFu::Inflator::DateTime> will 
