@@ -4,9 +4,9 @@ use strict;
 use base 'HTML::FormFu::Element::_Field';
 use Class::C3;
 
-use HTML::FormFu::Util qw/ process_attrs /;
+use HTML::FormFu::Util qw( process_attrs );
 
-__PACKAGE__->mk_attr_accessors(qw/ cols rows /);
+__PACKAGE__->mk_attr_accessors(qw( cols rows ));
 
 sub new {
     my $self = shift->next::method(@_);
@@ -51,7 +51,8 @@ sub _string_field {
 
     my $html = sprintf qq{<textarea name="%s"%s>},
         $render->{nested_name},
-        process_attrs( $render->{attributes} );
+        process_attrs( $render->{attributes} ),
+        ;
 
     if ( defined $render->{value} ) {
         $html .= $render->{value};
@@ -86,7 +87,7 @@ Is a sub-class of, and inherits methods from
 L<HTML::FormFu::Element::_Field>, 
 L<HTML::FormFu::Element>
 
-L<HTML::FormFu::FormFu>
+L<HTML::FormFu>
 
 =head1 AUTHOR
 
