@@ -2,12 +2,23 @@ package HTML::FormFu::Util;
 
 use strict;
 
+use HTML::FormFu::Constants qw( $SPACE );
 use HTML::FormFu::Literal;
 use Scalar::Util qw( blessed );
+use Readonly;
 use Exporter qw/ import /;
 use Carp qw/ croak /;
 
-our @EXPORT_OK = qw/
+Readonly my $EMPTY_STR => q{};
+Readonly my $SPACE     => q{ };
+
+our $LAST_SUB = $EMPTY_STR;
+
+our @EXPORT_OK = qw(
+    DEBUG
+    DEBUG_PROCESS
+    DEBUG_CONSTRAINTS
+    debug
     append_xml_attribute
     has_xml_attribute
     remove_xml_attribute
@@ -20,7 +31,73 @@ our @EXPORT_OK = qw/
     process_attrs
     split_name
     _merge_hashes
-    /;
+);
+
+# the empty prototype () means that when false, all debugging calls
+# will be optimised out during compilation
+
+sub DEBUG {
+    $ENV{HTML_FORMFU_DEBUG} || 0;
+}
+
+sub DEBUG_PROCESS () {
+    DEBUG
+        || $ENV{HTML_FORMFU_DEBUG_PROCESS}
+        || 0;
+}
+
+sub DEBUG_CONSTRAINTS {
+    DEBUG
+        || DEBUG_PROCESS
+        || $ENV{HTML_FORMFU_DEBUG_CONSTRAINTS}
+        || 0;
+}
+
+sub debug {
+    my ($message) = @_;
+
+    my ( undef, undef, undef, $sub ) = caller(1);
+
+    require 'Data/Dumper.pm';
+
+    warn "\n" if $sub ne $LAST_SUB;
+
+    if ( @_ > 1 ) {
+        warn "$sub()\n" if $sub ne $LAST_SUB;
+
+        while (@_) {
+            my $key   = shift;
+            my $value = shift;
+
+            if ( ref $value ) {
+                $value = Data::Dumper::Dumper($value);
+                $value =~ s/^\$VAR1 = //;
+            }
+            else {
+                $value = "'$value'\n";
+            }
+
+            warn "$key: $value";
+        }
+    }
+    elsif ( ref $message ) {
+        warn "$sub()\n" if $sub ne $LAST_SUB;
+
+        $message = Data::Dumper::Dumper($message);
+        $message =~ s/^\$VAR1 = /        /;
+
+        warn "$message\n";
+    }
+    else {
+        warn "$sub\n" if $sub ne $LAST_SUB;
+
+        warn "$message\n";
+    }
+
+    $LAST_SUB = $sub;
+
+    return;
+}
 
 sub _filter_components {
     my ( $args, $components ) = @_;
@@ -32,11 +109,12 @@ sub _filter_components {
 
         my $value;
 
-        @$components = grep {
+        @$components
+            = grep {
                    $_->can($name)
                 && defined( $value = $_->$name )
                 && $value eq $args->{$name}
-        } @$components;
+            } @$components;
     }
 
     return $components;
@@ -68,14 +146,20 @@ sub append_xml_attribute {
 
     if ( exists $attrs->{$key} && defined $attrs->{$key} ) {
         my $orig = 'string';
-        $orig = 'literal'
-            if blessed $attrs->{$key}
-                && $attrs->{$key}->isa('HTML::FormFu::Literal');
+
+        if ( blessed $attrs->{$key}
+            && $attrs->{$key}->isa('HTML::FormFu::Literal') )
+        {
+            $orig = 'literal';
+        }
 
         my $new = 'string';
-        $new = 'literal'
-            if blessed $value
-                && $value->isa('HTML::FormFu::Literal');
+
+        if ( blessed $value
+            && $value->isa('HTML::FormFu::Literal') )
+        {
+            $new = 'literal';
+        }
 
         $attrs->{$key} = $dispatcher{$orig}->{$new}->( $attrs->{$key}, $value );
     }
@@ -89,13 +173,24 @@ sub append_xml_attribute {
 sub _append_subs {
     return (
         literal => {
-            string => sub { $_[0]->push( xml_escape(" $_[1]") ); return $_[0] },
-            literal => sub { $_[0]->push(" $_[1]"); return $_[0] },
+            string => sub {
+                $_[0]->push( xml_escape(" $_[1]") );
+                return $_[0];
+            },
+            literal => sub {
+                $_[0]->push(" $_[1]");
+                return $_[0];
+            },
         },
         string => {
-            string => sub { $_[0] .= " $_[1]"; return $_[0] },
-            literal =>
-                sub { $_[1]->unshift( xml_escape("$_[0] ") ); return $_[1] },
+            string => sub {
+                $_[0] .= " $_[1]";
+                return $_[0];
+            },
+            literal => sub {
+                $_[1]->unshift( xml_escape("$_[0] ") );
+                return $_[1];
+            },
         },
     );
 }
@@ -110,14 +205,20 @@ sub has_xml_attribute {
 
     if ( exists $attrs->{$key} && defined $attrs->{$key} ) {
         my $orig = 'string';
-        $orig = 'literal'
-            if blessed $attrs->{$key}
-                && $attrs->{$key}->isa('HTML::FormFu::Literal');
+
+        if ( blessed $attrs->{$key}
+            && $attrs->{$key}->isa('HTML::FormFu::Literal') )
+        {
+            $orig = 'literal';
+        }
 
         my $new = 'string';
-        $new = 'literal'
-            if blessed $value
-                && $value->isa('HTML::FormFu::Literal');
+
+        if ( blessed $value
+            && $value->isa('HTML::FormFu::Literal') )
+        {
+            $new = 'literal';
+        }
 
         return $dispatcher{$orig}->{$new}->( $attrs->{$key}, $value );
     }
@@ -175,14 +276,20 @@ sub remove_xml_attribute {
 
     if ( exists $attrs->{$key} && defined $attrs->{$key} ) {
         my $orig = 'string';
-        $orig = 'literal'
-            if blessed $attrs->{$key}
-                && $attrs->{$key}->isa('HTML::FormFu::Literal');
+
+        if ( blessed $attrs->{$key}
+            && $attrs->{$key}->isa('HTML::FormFu::Literal') )
+        {
+            $orig = 'literal';
+        }
 
         my $new = 'string';
-        $new = 'literal'
-            if blessed $value
-                && $value->isa('HTML::FormFu::Literal');
+
+        if ( blessed $value
+            && $value->isa('HTML::FormFu::Literal') )
+        {
+            $new = 'literal';
+        }
 
         $attrs->{$key} = $dispatcher{$orig}->{$new}->( $attrs->{$key}, $value );
     }
@@ -234,22 +341,19 @@ sub _remove_subs {
 }
 
 sub _parse_args {
-    my %args = ();
 
     if ( !@_ ) {
-        %args = ();
+        return;
     }
     elsif ( @_ > 1 ) {
-        %args = @_;
+        return @_;
     }
     elsif ( ref $_[0] ) {
-        %args = %{ $_[0] };
+        return %{ $_[0] };
     }
     else {
-        %args = ( name => $_[0] );
+        return ( name => $_[0] );
     }
-
-    return %args;
 }
 
 sub require_class {
@@ -277,9 +381,11 @@ sub xml_escape {
 
     if ( ref $val eq 'HASH' ) {
         my %val = %$val;
-        for my $key ( keys %val ) {
-            $val{$key} = xml_escape( $val{$key} );
+
+        while ( my ( $key, $value ) = each %val ) {
+            $val{$key} = xml_escape($value);
         }
+
         return \%val;
     }
     elsif ( ref $val eq 'ARRAY' ) {
@@ -313,13 +419,24 @@ sub process_attrs {
     my ($attrs) = @_;
 
     eval { my %attrs = %$attrs };
-    croak $@ if $@;
+    croak 'argument to process_attrs() must be a hashref' if $@;
 
-    my $xml = join " ", map { sprintf qq{%s="%s"}, $_, $attrs->{$_} }
-        sort keys %$attrs;
+    my @attribute_parts;
 
-    $xml = " $xml"
-        if length $xml;
+    for my $attribute ( sort keys %$attrs ) {
+        my $value
+            = defined $attrs->{$attribute}
+            ? $attrs->{$attribute}
+            : $EMPTY_STR;
+
+        push @attribute_parts, sprintf '%s="%s"', $attribute, $value;
+    }
+
+    my $xml = join $SPACE, @attribute_parts;
+
+    if ( length $xml ) {
+        $xml = " $xml";
+    }
 
     return $xml;
 }
@@ -329,11 +446,12 @@ sub split_name {
 
     croak "split_name requires 1 arg" if @_ != 1;
 
-    return () if !defined $name;
+    return if !defined $name;
 
     if ( $name =~ /^ \w+ \[ /x ) {
 
         # copied from Catalyst::Plugin::Params::Nested::Expander
+        # redistributed under the same terms as Perl
 
         return grep {defined} (
             $name =~ /
@@ -345,6 +463,7 @@ sub split_name {
     elsif ( $name =~ /\./ ) {
 
         # Copied from CGI::Expand
+        # redistributed under the same terms as Perl
 
         # m// splits on unescaped '.' chars. Can't fail b/c \G on next
         # non ./ * -> escaped anything -> non ./ *
@@ -365,26 +484,25 @@ sub split_name {
     return ($name);
 }
 
-# sub _merge_hashes copied from Catalyst::Utils::merge_hashes()
+# sub _merge_hashes originally copied from Catalyst::Utils::merge_hashes()
 # redistributed under the same terms as Perl
 
 sub _merge_hashes {
     my ( $lefthash, $righthash ) = @_;
 
-    return $lefthash unless defined $righthash;
+    return $lefthash if !defined $righthash || !keys %$righthash;
 
     my %merged = %$lefthash;
-    for my $key ( keys %$righthash ) {
-        my $right_ref = ( ref $righthash->{$key} || '' ) eq 'HASH';
-        my $left_ref
-            = ( ( exists $lefthash->{$key} && ref $lefthash->{$key} ) || '' ) eq
-            'HASH';
-        if ( $right_ref and $left_ref ) {
-            $merged{$key}
-                = _merge_hashes( $lefthash->{$key}, $righthash->{$key} );
+
+    while ( my ( $key, $value ) = each %$righthash ) {
+        my $is_right_ref = ref $value eq 'HASH';
+        my $is_left_ref = exists $lefthash->{$key} && ref $value eq 'HASH';
+
+        if ( $is_left_ref && $is_right_ref ) {
+            $merged{$key} = _merge_hashes( $lefthash->{$key}, $value );
         }
         else {
-            $merged{$key} = $righthash->{$key};
+            $merged{$key} = $value;
         }
     }
 
